"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Map package or folder name to a unique, non-spaced ID that can be used in dot file.
 */
class MapNameToId {
    constructor() {
        this.mapNameToId = new Map();
        // must be 1-based, since is used to get next graphviz color:
        this.nextId = 1;
    }
    getId(name) {
        if (!this.hasId(name)) {
            this.mapNameToId.set(name, this.nextId++);
        }
        const num = this.mapNameToId.get(name);
        return `P${num}`;
    }
    getIdOrThrow(name) {
        if (!this.hasId(name)) {
            throw new Error(`cannot find name in map: '${name}'`);
        }
        return this.getId(name);
    }
    getNumberOrThrow(name) {
        if (!this.hasId(name)) {
            throw new Error(`cannot find name in map: '${name}'`);
        }
        return this.mapNameToId.get(name);
    }
    hasId(name) {
        return this.mapNameToId.has(name);
    }
}
exports.MapNameToId = MapNameToId;
//# sourceMappingURL=MapNameToId.js.map