"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class PackageFilter {
    constructor(config) {
        this.invalidPaths = this.parseBlacklist(config.importBlacklist);
        this.whiteListPaths = this.parseWhitelist(config.importWhitelist);
        this.skipSubFolders = config.skipSubFolders;
    }
    parseBlacklist(list) {
        return list.split(",");
    }
    parseWhitelist(list) {
        return list.split(",").filter(entry => entry.length > 0);
    }
    passesBlacklist(importPath) {
        return this.invalidPaths.indexOf(importPath) < 0;
    }
    passesWhitelist(importPath) {
        return (this.whiteListPaths.length === 0 ||
            this.whiteListPaths.indexOf(importPath) >= 0);
    }
    canOutputSubfoldersOf(folder) {
        return this.passesWhitelist(folder.importPath);
    }
    get hasWhitelist() {
        return this.whiteListPaths.length > 0;
    }
    isImportPathOkForFolder(folder) {
        return this.passesBlacklist(folder.importPath);
    }
    isImportPathOkForSubFolder(folder) {
        return !this.skipSubFolders && this.passesBlacklist(folder.importPath);
    }
    isImportPathOkForEdges(importPath) {
        return this.passesBlacklist(importPath);
    }
}
exports.PackageFilter = PackageFilter;
//# sourceMappingURL=PackageFilter.js.map